/*
 * open.c
 * This file provides the open functions required by gnozip
 */

#include "common.h"
#include "callbacks.h"
#include "exit.h"
#include "tar.h"
#include "zip.h"
#include "gzip.h"
#include "bzip2.h"
#include "open.h"

/*
 * function: gz_filetype
 * input: A gchar string.
 * output: an integer representing the filetype
 */

int
gz_get_filetype (const gchar *filename)
{
  const char *mime_type = NULL;

  mime_type = gnome_mime_type_of_file (filename);
  DEBUG ("gz_get_filetype: ", mime_type);
  
  if (!(strcmp (mime_type, "application/x-tar")))
      return FILE_TAR;
  if (!(strcmp (mime_type, "application/x-gtar")))
      return FILE_TAR;
  if (!(strcmp (mime_type, "application/x-gzip")))
      return FILE_GZIP;
  if (!(strcmp (mime_type, "application/x-bzip2")))
  {
      if (strstr (filename, ".tar.bz2"))
          return FILE_TBZ2;
      else
          return FILE_BZIP2;
  }
  if (!(strcmp (mime_type, "application/x-compressed-tar")))
      return FILE_TGZ;
/*  if (!(strcmp (mime_type, "application/x-bzip-compressed-tar")))
      return FILE_TBZ2; */
  if (!(strcmp (mime_type, "application/zip")))
      return FILE_ZIP;
  if (!(strcmp (mime_type, "application/x-arj")))
      return FILE_ARJ;
  if (!(strcmp (mime_type, "application/x-lha")))
      return FILE_LHA;
  if (!(strcmp (mime_type, "application/x-lzh")))
      return FILE_LHA;
  if (!(strcmp (mime_type, "application/x-rar")))
      return FILE_RAR;
  return FILE_UNKN;
}

/* function: gz_open_file_selection
 * 
 * Opens a file selection box and sets callbacks.
 */

void
gz_open_file_selection()
{
  GtkWidget *gz_filesel_box;

  gz_filesel_box = gtk_file_selection_new(_("GnoZip: Open File..."));
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (gz_filesel_box)->ok_button),
		      "clicked",
		      (GtkSignalFunc) on_open_filesel_activate,
		      gz_filesel_box );
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
			                             (gz_filesel_box)->cancel_button),
		  	     "clicked",
			     (GtkSignalFunc) gtk_widget_destroy,
                             GTK_OBJECT (gz_filesel_box));

  gtk_widget_show(gz_filesel_box);
}

void
gz_save_as_file_selection()
{
  GtkWidget *gz_filesel_box;

  gz_filesel_box = gtk_file_selection_new(_("GnoZip: Save File as..."));
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (gz_filesel_box)->ok_button),
		      "clicked",
		      (GtkSignalFunc) on_save_as_filesel_activate,
		      gz_filesel_box );
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
					 	(gz_filesel_box)->cancel_button),
			    		"clicked",
					(GtkSignalFunc) gtk_widget_destroy,
					GTK_OBJECT (gz_filesel_box));

  gtk_widget_show(gz_filesel_box);
}

/* function: gz_open
 * input: A filename as gchar string.
 * output: error code.
 * 
 * This function accept a filename as input, try to determinate the file type,
 * and accordingly lists in the main window the compressed file contents.
 */

int
gz_open (const gchar *filename)
{
  GtkWidget *main_clist = NULL;
  GList *current = NULL;
  int row;

  DEBUG("gz_open:",filename);

  main_clist = gz_mainclist_get_widget ();

  filetype = gz_get_filetype (filename);

  if (filelist) gz_clear_filelist (&filelist, GTK_CLIST (main_clist));
  
  switch (filetype)
  {
   case FILE_ZIP:
     filelist = gz_zip_list (filename);
     fileopen = TRUE;
     break;
   case FILE_GZIP:
     filelist = gz_gzip_list (filename);
     fileopen = TRUE;
     break;
   case FILE_BZIP2:
     filelist = gz_bzip2_list (filename);
     fileopen = TRUE;
     break;
   case FILE_TGZ:
     filelist = gz_tar_list (filename, 1);
     fileopen = TRUE;
     break;
   case FILE_TBZ2:
     filelist = gz_tar_list (filename, 2);
     fileopen = TRUE;
     break;
   case FILE_TAR:
     filelist = gz_tar_list (filename, 0);
     fileopen = TRUE;
     break;
   case FILE_ARJ:
     ERROR ("gz_open:", _("File type not yet supported!"))
     break;
   case FILE_LHA:
     ERROR ("gz_open:", _("File type not yet supported!"))
     break;
   case FILE_RAR:
     ERROR ("gz_open:", _("File type not yet supported!"))
     break;
   case FILE_UNKN:
     ERROR ("gz_open:", _("Unrecognized file type!"))
   default:;
  }

  row = 0;
  current = filelist;
  gtk_clist_freeze ((GtkCList *)main_clist);
  while (current)
  {
    gchar *perms = (((gz_list_entry *)(current->data))->perms);
    gtk_clist_append ((GtkCList *)main_clist, current->data);
    if (perms)
    {
	    if (*perms == 'd')
	    {
		    gtk_clist_set_pixmap ((GtkCList *)main_clist,
					  row,
					  COL_ICON,
					  dir_icon_pixmap,
					  dir_icon_mask);
	    }
	    if (*perms == 'l')
	    {
		    gtk_clist_set_pixmap ((GtkCList *)main_clist,
					  row,
					  COL_ICON,
					  link_icon_pixmap,
					  link_icon_mask);
	    }
    }
    row++;
    current = current->next;
  }
  gtk_clist_thaw ((GtkCList *)main_clist);
  flnum = g_list_length (filelist);
  return 0;
}

